from django.db.models import BooleanField, DateTimeField, DateField, TimeField
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from django.contrib import messages, admin
from django.utils.translation import ngettext


class AutoActionsMixin:
    def get_actions(self, request):
        actions = super().get_actions(request)
        auto_actions = self._get_auto_actions()
        actions.update(auto_actions)
        return actions

    def _get_auto_actions(self):
        auto_actions = {}
        exclude_fields = getattr(self, "exclude_auto_actions", [])

        def create_action(field_name, value, display_value):
            def action(modeladmin, request, queryset):
                updated_count = queryset.update(**{field_name: value})
                model_name = modeladmin.model._meta.verbose_name
                model_name_plural = modeladmin.model._meta.verbose_name_plural
                messages.success(
                    request,
                    ngettext(
                        f"Successfully updated {updated_count} {model_name} to {field_name} = {display_value}",
                        f"Successfully updated {updated_count} {model_name_plural} to {field_name} = {display_value}",
                        updated_count,
                    ),
                )

            return action

        for field in (
            f for f in self.model._meta.fields if f.name not in exclude_fields
        ):
            field_name = field.name

            if isinstance(field, BooleanField):
                possible_states = [True, False]
                if field.null:
                    possible_states.append(None)

                for state in possible_states:
                    action_name = f"set_{field_name}_{state}"
                    auto_actions[action_name] = (
                        create_action(field_name, state, state),
                        action_name,
                        _(f"Set {field_name} to {state}"),
                    )

            elif isinstance(field, DateTimeField):
                action_name = f"set_{field_name}_now"
                auto_actions[action_name] = (
                    create_action(field_name, timezone.now(), "now"),
                    action_name,
                    _(f"Set {field_name} to now"),
                )

            elif isinstance(field, DateField):
                action_name = f"set_{field_name}_today"
                auto_actions[action_name] = (
                    create_action(field_name, timezone.now().date(), "today"),
                    action_name,
                    _(f"Set {field_name} to today"),
                )

            elif isinstance(field, TimeField):
                action_name = f"set_{field_name}_now"
                auto_actions[action_name] = (
                    create_action(field_name, timezone.now().time(), "now"),
                    action_name,
                    _(f"Set {field_name} to now"),
                )

        return auto_actions


class AutoActionsModelAdmin(AutoActionsMixin, admin.ModelAdmin):
    pass



from pathlib import Path
from setuptools import setup, find_packages

VERSION = "0.1.3"
DESCRIPTION = "Automatically generates Django admin actions based on your model's fields"
this_directory = Path(__file__).parent
LONG_DESCRIPTION = (this_directory / "README.md").read_text()

setup(
    name='django-auto-actions',
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    version=VERSION,
    author='Félix Gravel',
    author_email="felix.gravel@tlmgo.com",
    url="https://github.com/Flexonze/django-auto-actions",
    packages=find_packages(),
    include_package_data=True,
    python_requires=">=3.10",
    install_requires=[
        'Django>=4.2',
    ],
    license="MIT",
    classifiers=[
      "License :: OSI Approved :: MIT License",
      "Intended Audience :: Developers",
      "Operating System :: OS Independent",
      "Programming Language :: Python",
      "Programming Language :: Python :: 3.10",
      "Programming Language :: Python :: 3.11",
      "Programming Language :: Python :: 3.12",
      "Topic :: Software Development :: Libraries :: Python Modules",
      "Framework :: Django",
      "Framework :: Django :: 4.2",
      "Framework :: Django :: 5.1",
    ]
)
